use Test::More tests => 6;

use strict;
use warnings;

use DE_EPAGES::Core::API::Script qw ( RunScript );
use DE_EPAGES::Database::API::Connection qw ( GetTestSite RunOnStore );
use DE_EPAGES::Object::API::Factory qw (
    ExistsObjectByPath
    LoadClassByAlias
    LoadObject
    LoadObjectByPath
    LoadRootObject
);
use DE_EPAGES::Order::API::Order qw ( CreateOrderWithContainer );
use DE_EPAGES::Trigger::API::Trigger qw (TriggerHook);
use DE_EPAGES::Customer::API::Shop qw ( GetNextCustomerNo GetCustomerFolder );
use DE_EPAGES::CrossSelling::API::AutoCrossSelling qw (
    UpdateAutoCrossSellingViaScheduler
);
use DE_EPAGES::CrossSelling::API::Table::AutoCrossSelling qw ( DeleteAutoCrossSelling );

use Readonly;
Readonly::Scalar my $ROOTPATH  => '/Test';
Readonly::Scalar my $ShopAlias => 'TestShop';

#========================================================================================
# description  This subroutine tests Shop->get('AutoCrossSellings')
#========================================================================================
sub TestAutoCrossSellings {
    # (1) Create a new shop with three products
    my $ParentObject = LoadObjectByPath($ROOTPATH);
    my $Shop = $ParentObject->insertChild('Shop', {
        Alias => $ShopAlias
    });
    ok( $Shop->exists, "inserted $ShopAlias" );

    $Shop->featureMaxValue('Products', 3);
    $Shop->featureMaxValue('AutoCrossSelling', 1);
    $Shop->set({ AutoCrossSellingCollectData => 1 });

    my %hProducts;
    foreach (qw( Product1 Product2 )) {
        ## no critic qw(ProhibitMagicNumbers)
        $hProducts{$_} = _InsertProduct($Shop, $_, '10.0');
        ## use critic
    }


    # (2) Add auto cross-selling information via an order
    my $CustomerNo = GetNextCustomerNo($Shop);
    my $CustomerFolder = GetCustomerFolder($Shop);
    my $Customer = LoadClassByAlias('Customer')->insertObject({
        Parent => $CustomerFolder,
        Alias  => $CustomerNo,
    });
    my $Order = CreateOrderWithContainer($Customer);
    my $Container = $Order->get('LineItemContainer');
    $hProducts{$_}->addToContainer($Container, { Quantity => 1 }) foreach (keys %hProducts);
    TriggerHook('API_OrderCreated', { Order => $Order });
    UpdateAutoCrossSellingViaScheduler();


    # (3) Retrieve auto cross-selling information via $Shop->get('AutoCrossSellings')
    my $Iterator = $Shop->get('AutoCrossSellings');
    my %hProductsCopy = %hProducts;
    while (my $hInfoAutoCrossSelling = <$Iterator>) {
        my $Product = LoadObject($hInfoAutoCrossSelling->{ProductID1});
        delete $hProductsCopy{$Product->alias};
    }
    ok( scalar keys %hProductsCopy == 0, 'all products found in auto cross-selling' );


    # (4) Delete previous inserted auto cross-selling products
    DeleteAutoCrossSelling($hProducts{Product1}->id, $hProducts{Product2}->id);
    DeleteAutoCrossSelling($hProducts{Product2}->id, $hProducts{Product1}->id);


    # (5) Retrieve no auto cross-selling products
    $Iterator = $Shop->get('AutoCrossSellings');
    my @aProducts;
    while (my $hInfoAutoCrossSelling = <$Iterator>) {
        push @aProducts, $hInfoAutoCrossSelling->{ProductID1}
            if (defined $hInfoAutoCrossSelling);
    }
    ok( scalar @aProducts == 0, 'no further products found' );

    return;
}

sub _InsertProduct {
    # my $self = shift;
    my ($Shop, $Alias, $Price) = @_;

    my $ProductFolder = $Shop->folder('Products');
    my $NewProduct = $ProductFolder->insertChild('Product', {
       Alias => $Alias,
       Shop  => $Shop,
    });
    $NewProduct->setListPrice('GBP', 0, $Price);

    return $NewProduct;
}

sub Init {
    Cleanup();

    LoadRootObject()->insertChild('Object', {
        Alias => 'Test'
    });

    ok( 1, 'init' );
    return;
}

sub Cleanup {
    if (ExistsObjectByPath($ROOTPATH)) {
        LoadObjectByPath($ROOTPATH)->delete;
    }

    ok( 1, 'cleanup' );
    return;
}

RunScript(
    Sub => sub {
        RunOnStore(
            Store => GetTestSite(),
            Sub => sub {
                Init();

                TestAutoCrossSellings();

                Cleanup();
            }
        )
    },
    Debug => 1,
);

1;
